"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const objectHash = require("object-hash");
exports.OctokitPlugin = (cache, logger = null) => {
    const trace = (obj, msg) => {
        if (logger !== null && typeof logger.child === 'function' && typeof logger.trace === 'function') {
            logger.child({ component: 'octokit/plugin/cache' }).trace(obj, msg);
        }
    };
    return (octokit) => {
        octokit.hook.error('request', async (err, options) => {
            if (err.code === 304) {
                return await cache.getCache(options);
            }
            else {
                throw err;
            }
        });
        octokit.hook.before('request', async (options) => {
            trace({ options }, 'before: request - options');
            if (await cache.inCache(options) === true) {
                const etag = await cache.getEtag(options);
                options.headers['If-None-Match'] = etag;
                trace({ etag }, 'request: etag header set');
            }
        });
        octokit.hook.after('request', async (result, options) => {
            trace({ options }, 'after: request - options');
            if (result.status !== 304) {
                trace({ result }, 'after: status not 304 - caching results');
                await cache.putCache(options, result);
            }
        });
    };
};
class OctokitCache {
    constructor(cache, options = { prefix: 'cache', separator: ':' }, logger = null) {
        this.prefix = options.prefix;
        this.separator = options.separator;
        if (this.prefix) {
            this.prefix += this.separator;
        }
        this.cache = cache;
        this.logger = logger;
    }
    hashIt(options) {
        const opts = lodash_1.omit(lodash_1.cloneDeep(options), ['headers', 'request']);
        opts.url = opts.url.replace(/access_token=[a-z0-9]{40}/ig, '').replace(/\?$/ig, '');
        const hash = objectHash(opts);
        this.trace({ hash }, 'hashIt');
        return hash;
    }
    async inCache(options) {
        const hash = this.hashIt(options);
        const exists = await this.cache.exists(`${this.prefix}${hash}${this.separator}etag`);
        this.trace({ hash, exists }, `inCache - ${exists}`);
        return Promise.resolve(exists);
    }
    async getEtag(options) {
        const hash = this.hashIt(options);
        const etag = await this.cache.get(`${this.prefix}${hash}${this.separator}etag`);
        this.trace({ hash, etag }, 'getEtag');
        return Promise.resolve(etag);
    }
    async getCache(options) {
        const hash = this.hashIt(options);
        const data = await this.cache.get(`${this.prefix}${hash}${this.separator}data`);
        const headers = await this.cache.get(`${this.prefix}${hash}${this.separator}headers`);
        headers.status = '304 Not Modified';
        const status = 304;
        this.trace({ hash, status }, 'getCache');
        return Promise.resolve({
            data,
            headers,
            status,
        });
    }
    async putCache(options, results) {
        const hash = this.hashIt(options);
        await this.cache.set(`${this.prefix}${hash}${this.separator}etag`, results.headers.etag);
        await this.cache.set(`${this.prefix}${hash}${this.separator}data`, results.data);
        await this.cache.set(`${this.prefix}${hash}${this.separator}headers`, results.headers);
        this.trace({ hash, prefix: `${this.prefix}${hash}${this.separator}` }, 'putCache');
        return Promise.resolve(true);
    }
    trace(obj, msg) {
        if (this.logger !== null && typeof this.logger.child === 'function' && typeof this.logger.trace === 'function') {
            this.logger.child({ component: 'octokit/plugin/cache' }).trace(obj, msg);
        }
    }
}
exports.OctokitCache = OctokitCache;
//# sourceMappingURL=cache.js.map